@@ -25,8 +25,6 @@
    Caleb Moore <c.moore@student.unsw.edu.au>
 */
 
-#include "config.h"
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <glib.h>
@@ -46,19 +44,9 @@ rsvg_cairo_render_free (RsvgRender * self)
 {
     RsvgCairoRender *me = RSVG_CAIRO_RENDER (self);
 
-    /* TODO */
-
-#ifdef HAVE_PANGOFT2
-    if (me->font_config_for_testing) {
-        FcConfigDestroy (me->font_config_for_testing);
-        me->font_config_for_testing = NULL;
-    }
-
-    if (me->font_map_for_testing) {
-        g_object_unref (me->font_map_for_testing);
-        me->font_map_for_testing = NULL;
-    }
-#endif
+    g_assert (me->cr_stack == NULL);
+    g_assert (me->bb_stack == NULL);
+    g_assert (me->surfaces_stack == NULL);
 
     g_free (me);
 }
@@ -74,7 +62,7 @@ rsvg_cairo_render_new (cairo_t * cr, double width, double height)
     cairo_render->super.create_pango_context = rsvg_cairo_create_pango_context;
     cairo_render->super.render_pango_layout = rsvg_cairo_render_pango_layout;
     cairo_render->super.render_surface = rsvg_cairo_render_surface;
-    cairo_render->super.render_path = rsvg_cairo_render_path;
+    cairo_render->super.render_path_builder = rsvg_cairo_render_path_builder;
     cairo_render->super.pop_discrete_layer = rsvg_cairo_pop_discrete_layer;
     cairo_render->super.push_discrete_layer = rsvg_cairo_push_discrete_layer;
     cairo_render->super.add_clipping_rect = rsvg_cairo_add_clipping_rect;
@@ -89,11 +77,6 @@ rsvg_cairo_render_new (cairo_t * cr, double width, double height)
     cairo_render->bb_stack = NULL;
     cairo_render->surfaces_stack = NULL;
 
-#ifdef HAVE_PANGOFT2
-    cairo_render->font_config_for_testing = NULL;
-    cairo_render->font_map_for_testing = NULL;
-#endif
-
     cairo_matrix_init_identity (&matrix);
     rsvg_bbox_init (&cairo_render->bbox, &matrix);
 
@@ -174,7 +157,6 @@ rsvg_cairo_new_drawing_ctx (cairo_t * cr, RsvgHandle * handle)
     draw->dpi_y = handle->priv->dpi_y;
     draw->vb.rect.width = data.em;
     draw->vb.rect.height = data.ex;
-    draw->num_elements_acquired = 0;
     draw->pango_context = NULL;
     draw->drawsub_stack = NULL;
     draw->acquired_nodes = NULL;
@@ -220,11 +202,10 @@ rsvg_handle_render_cairo_sub (RsvgHandle * handle, cairo_t * cr, const char *id)
 {
     RsvgDrawingCtx *draw;
     RsvgNode *drawsub = NULL;
-    gboolean retval = FALSE;
 
     g_return_val_if_fail (handle != NULL, FALSE);
 
-    if (handle->priv->state != RSVG_HANDLE_STATE_CLOSED_OK)
+    if (!handle->priv->finished)
         return FALSE;
 
     if (id && *id)
@@ -241,25 +222,18 @@ rsvg_handle_render_cairo_sub (RsvgHandle * handle, cairo_t * cr, const char *id)
 
     while (drawsub != NULL) {
         draw->drawsub_stack = g_slist_prepend (draw->drawsub_stack, drawsub);
-        drawsub = drawsub->parent;
+        drawsub = rsvg_node_get_parent (drawsub);
     }
 
-    rsvg_state_push (draw);
     cairo_save (cr);
 
-    rsvg_node_draw ((RsvgNode *) handle->priv->treebase, draw, 0);
-
-    if (rsvg_drawing_ctx_limits_exceeded (draw)) {
-        retval = FALSE;
-    } else {
-        retval = TRUE;
-    }
+    rsvg_node_draw (handle->priv->treebase, draw, 0);
 
     cairo_restore (cr);
-    rsvg_state_pop (draw);
+
     rsvg_drawing_ctx_free (draw);
 
-    return retval;
+    return TRUE;
 }
 
 /**
