@@ -33,7 +33,6 @@
 
 struct _RsvgDefs {
     GHashTable *hash;
-    GPtrArray *unnamed;
     GHashTable *externs;
     RsvgHandle *ctx;
 };
@@ -46,56 +45,57 @@ rsvg_defs_new (RsvgHandle *handle)
     result->hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
     result->externs =
         g_hash_table_new_full (g_str_hash, g_str_equal, g_free, (GDestroyNotify) g_object_unref);
-    result->unnamed = g_ptr_array_new ();
     result->ctx = handle; /* no need to take a ref here */
 
     return result;
 }
 
-static RsvgHandle *
-rsvg_defs_load_extern (const RsvgDefs * defs, const char *uri)
+static int
+rsvg_defs_load_extern (const RsvgDefs * defs, const char *name)
 {
-    RsvgHandle *handle = NULL;
+    RsvgHandle *handle;
+    gchar *filename, *base_uri;
     char *data;
     gsize data_len;
+    gboolean rv;
 
-    data = _rsvg_handle_acquire_data (defs->ctx, uri, NULL, &data_len, NULL);
+    filename = _rsvg_io_get_file_path (name, rsvg_handle_get_base_uri (defs->ctx));
+
+    data = _rsvg_handle_acquire_data (defs->ctx, name, NULL, &data_len, NULL);
 
     if (data) {
         handle = rsvg_handle_new ();
-        rsvg_handle_set_base_uri (handle, uri);
 
-        if (rsvg_handle_write (handle, (guchar *) data, data_len, NULL)
-            && rsvg_handle_close (handle, NULL)) {
-            g_hash_table_insert (defs->externs, g_strdup (uri), handle);
-        } else {
-            g_object_unref (handle);
-            handle = NULL;
+        base_uri = rsvg_get_base_uri_from_filename (filename);
+        rsvg_handle_set_base_uri (handle, base_uri);
+        g_free (base_uri);
+
+        rv = rsvg_handle_write (handle, (guchar *) data, data_len, NULL);
+        rv = rsvg_handle_close (handle, NULL) && rv;
+        if (rv) {
+            g_hash_table_insert (defs->externs, g_strdup (name), handle);
         }
 
         g_free (data);
     }
 
-    return handle;
+    g_free (filename);
+    return 0;
 }
 
 static RsvgNode *
-rsvg_defs_extern_lookup (const RsvgDefs * defs, const char *possibly_relative_uri, const char *name)
+rsvg_defs_extern_lookup (const RsvgDefs * defs, const char *filename, const char *name)
 {
-    RsvgHandle *handle;
-    char *uri;
-
-    uri = rsvg_handle_resolve_uri (defs->ctx, possibly_relative_uri);
-    if (!uri)
-        return NULL;
-
-    handle = (RsvgHandle *) g_hash_table_lookup (defs->externs, uri);
-    if (handle == NULL) {
-        handle = rsvg_defs_load_extern (defs, uri);
+    RsvgHandle *file;
+    file = (RsvgHandle *) g_hash_table_lookup (defs->externs, filename);
+    if (file == NULL) {
+        if (rsvg_defs_load_extern (defs, filename))
+            return NULL;
+        file = (RsvgHandle *) g_hash_table_lookup (defs->externs, filename);
     }
 
-    if (handle != NULL)
-        return g_hash_table_lookup (handle->priv->defs->hash, name);
+    if (file != NULL)
+        return g_hash_table_lookup (file->priv->defs->hash, name);
     else
         return NULL;
 }
@@ -121,43 +121,26 @@ rsvg_defs_lookup (const RsvgDefs * defs, const char *name)
 }
 
 void
-rsvg_defs_set (RsvgDefs * defs, const char *name, RsvgNode * val)
+rsvg_defs_register_node_by_id (RsvgDefs *defs, const char *id, RsvgNode *node)
 {
-    if (name == NULL);
-    else if (name[0] == '\0');
-    else
-        rsvg_defs_register_name (defs, name, val);
-    rsvg_defs_register_memory (defs, val);
-}
+    g_assert (defs != NULL);
+    g_assert (id != NULL);
+    g_assert (node != NULL);
 
-void
-rsvg_defs_register_name (RsvgDefs * defs, const char *name, RsvgNode * val)
-{
-    if (g_hash_table_lookup (defs->hash, name))
+    if (g_hash_table_lookup (defs->hash, id))
         return;
 
-    g_hash_table_insert (defs->hash, g_strdup (name), val);
-}
-
-void
-rsvg_defs_register_memory (RsvgDefs * defs, RsvgNode * val)
-{
-    g_ptr_array_add (defs->unnamed, val);
+    g_hash_table_insert (defs->hash, g_strdup (id), node);
 }
 
 void
 rsvg_defs_free (RsvgDefs * defs)
 {
-    guint i;
-
     g_hash_table_destroy (defs->hash);
-
-    for (i = 0; i < defs->unnamed->len; i++)
-        ((RsvgNode *) g_ptr_array_index (defs->unnamed, i))->
-            free (g_ptr_array_index (defs->unnamed, i));
-    g_ptr_array_free (defs->unnamed, TRUE);
+    defs->hash = NULL;
 
     g_hash_table_destroy (defs->externs);
+    defs->externs = NULL;
 
     g_free (defs);
 }
