@@ -37,7 +37,7 @@
 G_BEGIN_DECLS 
 
 G_GNUC_INTERNAL
-RsvgNode *rsvg_new_image (void);
+RsvgNode *rsvg_new_image (const char *element_name);
 
 typedef struct _RsvgNodeImage RsvgNodeImage;
 
@@ -52,8 +52,6 @@ G_GNUC_INTERNAL
 void rsvg_preserve_aspect_ratio (unsigned int aspect_ratio, double width,
                                  double height, double *w, double *h, double *x, double *y);
 G_GNUC_INTERNAL
-gchar *rsvg_get_file_path (const gchar * filename, const gchar * basedir);
-G_GNUC_INTERNAL
 cairo_surface_t *rsvg_cairo_surface_new_from_href (RsvgHandle *handle, const char *href, GError ** error);
 
 G_END_DECLS
