@@ -37,17 +37,17 @@
 G_BEGIN_DECLS 
 
 G_GNUC_INTERNAL
-RsvgNode *rsvg_new_use (void);
+RsvgNode *rsvg_new_use (const char *element_name);
 G_GNUC_INTERNAL
-RsvgNode *rsvg_new_symbol (void);
+RsvgNode *rsvg_new_symbol (const char *element_name);
 G_GNUC_INTERNAL
-RsvgNode *rsvg_new_svg (void);
+RsvgNode *rsvg_new_svg (const char *element_name);
 G_GNUC_INTERNAL
-RsvgNode *rsvg_new_defs (void);
+RsvgNode *rsvg_new_defs (const char *element_name);
 G_GNUC_INTERNAL
-RsvgNode *rsvg_new_group (void);
+RsvgNode *rsvg_new_group (const char *element_name);
 G_GNUC_INTERNAL
-RsvgNode *rsvg_new_switch (void);
+RsvgNode *rsvg_new_switch (const char *element_name);
 
 typedef struct _RsvgNodeGroup RsvgNodeGroup;
 typedef struct _RsvgNodeUse RsvgNodeUse;
@@ -56,7 +56,6 @@ typedef struct _RsvgNodeSvg RsvgNodeSvg;
 
 struct _RsvgNodeGroup {
     RsvgNode super;
-    char *name;
 };
 
 struct _RsvgNodeSymbol {
@@ -80,19 +79,19 @@ struct _RsvgNodeSvg {
 };
 
 G_GNUC_INTERNAL
-void rsvg_pop_def_group     (RsvgHandle * ctx);
-G_GNUC_INTERNAL
-void rsvg_node_group_pack   (RsvgNode * self, RsvgNode * child);
+void rsvg_node_add_child   (RsvgNode * self, RsvgNode * child);
 G_GNUC_INTERNAL
 void rsvg_node_draw         (RsvgNode * self, RsvgDrawingCtx * ctx, int dominate);
 G_GNUC_INTERNAL
 void _rsvg_node_draw_children   (RsvgNode * self, RsvgDrawingCtx * ctx, int dominate);
 G_GNUC_INTERNAL
-void _rsvg_node_finalize    (RsvgNode * self);
-G_GNUC_INTERNAL
 void _rsvg_node_free        (RsvgNode * self);
+
 G_GNUC_INTERNAL
-void _rsvg_node_init        (RsvgNode * self, RsvgNodeType type);
+void _rsvg_node_init        (RsvgNode * self,
+                             RsvgNodeType type,
+                             RsvgNodeVtable *vtable);
+
 G_GNUC_INTERNAL
 void _rsvg_node_svg_apply_atts  (RsvgNodeSvg * self, RsvgHandle * ctx);
 
