@@ -180,7 +180,7 @@ is_svg_or_subdir (GFile *file)
     result = FALSE;
 
     basename = g_file_get_basename (file);
-    ignore = g_str_has_prefix (basename, "ignore") || strcmp (basename, "resources") == 0;
+    ignore = g_str_has_prefix (basename, "ignore");
 
     if (ignore)
 	goto out;
@@ -225,8 +225,6 @@ read_png (const char *test_name)
 
   reference_uri = g_strconcat (test_name, "-ref.png", NULL);
   file = g_file_new_for_uri (reference_uri);
-  g_free (reference_uri);
-
   stream = g_file_read (file, NULL, &error);
   g_assert_no_error (error);
   g_assert (stream);
@@ -306,7 +304,6 @@ rsvg_cairo_check (gconstpointer data)
     cairo_destroy (cr);
 
     g_object_unref (rsvg);
-    g_free (test_file_base);
 }
 
 int
@@ -324,7 +321,7 @@ main (int argc, char **argv)
 
         base = g_file_new_for_path (test_utils_get_test_data_path ());
         tests = g_file_get_child (base, "reftests");
-        test_utils_add_test_for_all_files ("/rsvg/reftest", tests, tests, rsvg_cairo_check, is_svg_or_subdir);
+        test_utils_add_test_for_all_files ("/rsvg-test/reftests", tests, tests, rsvg_cairo_check, is_svg_or_subdir);
         g_object_unref (tests);
         g_object_unref (base);
     } else {
@@ -333,7 +330,7 @@ main (int argc, char **argv)
         for (i = 1; i < argc; i++) {
             GFile *file = g_file_new_for_commandline_arg (argv[i]);
 
-            test_utils_add_test_for_all_files ("/rsvg/reftest", NULL, file, rsvg_cairo_check, is_svg_or_subdir);
+            test_utils_add_test_for_all_files ("/rsvg-test/reftests", NULL, file, rsvg_cairo_check, is_svg_or_subdir);
 
             g_object_unref (file);
         }
